@@ -62,7 +62,7 @@ fn combined_stream(streams: Vec<String>) -> String { streams.join("/") }
 
 pub struct WebSockets<'a, WE> {
     pub socket: Option<(WebSocketStream<MaybeTlsStream<TcpStream>>, Response)>,
-    handler: Box<dyn FnMut(WE) -> Result<()> + 'a>,
+    handler: Box<dyn FnMut(WE) -> Result<()> + 'a + Send>,
     conf: Config,
 }
 
@@ -72,7 +72,7 @@ impl<'a, WE: serde::de::DeserializeOwned> WebSockets<'a, WE> {
     /// see examples/binance_websockets.rs
     pub fn new<Callback>(handler: Callback) -> WebSockets<'a, WE>
     where
-        Callback: FnMut(WE) -> Result<()> + 'a,
+        Callback: FnMut(WE) -> Result<()> + 'a + Send,
     {
         Self::new_with_options(handler, Config::default())
     }
@@ -82,7 +82,7 @@ impl<'a, WE: serde::de::DeserializeOwned> WebSockets<'a, WE> {
     /// see examples/binance_websockets.rs
     pub fn new_with_options<Callback>(handler: Callback, conf: Config) -> WebSockets<'a, WE>
     where
-        Callback: FnMut(WE) -> Result<()> + 'a,
+        Callback: FnMut(WE) -> Result<()> + 'a + Send,
     {
         WebSockets {
             socket: None,
